@@ -117,11 +117,14 @@ setAndCalculateOne <- nimbleFunction(
 #' @param model An uncompiled or compiled NIMBLE model.  This argument is required.
 #' @param targetNodes A character vector containing the names of one or more nodes or variables in the model.  This argument is required.
 #' @author Daniel Turek
+#' @aliases setAndCalculateDiff
 #' @export
-#' @details Calling setAndCalculate(model, targetNodes) will return a function with a single, required argument:
+#' @details Calling \code{setAndCalculate(model, targetNodes)} or \code{setAndCalculate(model, targetNodes)} will return a nimbleFunction object whose \code{run} function takes a single, required argument:
 #' 
 #' targetValues: A vector of numeric values which will be put into the target nodes in the specified model object.  The length of this numeric vector much exactly match the number of target nodes.
 #'
+#' The difference between \code{setAndCalculate} and \code{setAndCalculateDiff} is the return value of their \code{run} functions.  In the former, \code{run} returns the sum of the log probabilities of the \code{targetNodes} with the provided \code{targetValues}, while the latter returns the difference between that sum with the new \code{targetValues} and the previous values in the \code{model}.
+#' 
 #' @examples
 #' code <- nimbleCode({ for(i in 1:3) { x[i] ~ dnorm(0,1); y[i] ~ dnorm(0, 1)}})
 #' Rmodel <- nimbleModel(code)
@@ -140,6 +143,8 @@ setAndCalculate <- nimbleFunction(
     }, where = getLoadingNamespace()
 )
 
+#' @rdname setAndCalculate
+#' @export
 setAndCalculateDiff <- nimbleFunction(
     setup = function(model, targetNodes) {
         targetNodesAsScalar <- model$expandNodeNames(targetNodes, returnScalarComponents = TRUE)
@@ -216,20 +221,54 @@ codeBlockClass <- setRefClass(
     
 )
 
+mcmc_generateControlListArgument <- function(requiredControlNames, samplerFunction, control, controlDefaults) {
+    if(missing(requiredControlNames)) {
+        if(is.character(samplerFunction)) {
+            if(exists(samplerFunction) && is.nfGenerator(eval(as.name(samplerFunction)))) {
+                samplerFunction <- eval(as.name(samplerFunction))
+            } else {
+                samplerFunction_type <- paste0('sampler_', samplerFunction)
+                if(exists(samplerFunction_type) && is.nfGenerator(eval(as.name(samplerFunction_type)))) {
+                    samplerFunction <- eval(as.name(samplerFunction_type))
+                } else stop(paste0('cannot find sampler type \'', type, '\''))
+            }
+        }
+        requiredControlNames <- mcmc_findControlListNamesInCode(samplerFunction)
+    }
+    if(missing(controlDefaults))
+        controlDefaults <- getNimbleOption('MCMCcontrolDefaultList')
+    if(missing(control))
+        control <- list()
+    thisControlList <- controlDefaults           ## start with all the defaults
+    thisControlList[names(control)] <- control   ## add in any controls provided as an argument
+    missingControlNames <- setdiff(requiredControlNames, names(thisControlList))
+    missingControlNames <- missingControlNames[!grepl('^dep_', missingControlNames)]   ## dependents for conjugate samplers are exempted from this check
+    if(length(missingControlNames) != 0)  stop(paste0('Required control names are missing for sampler: ', paste0(missingControlNames, collapse=', ')))
+    thisControlList <- thisControlList[requiredControlNames]
+    return(thisControlList)
+}
+
+
 
 mcmc_listContentsToStr <- function(ls) {
-    if(any(unlist(lapply(ls, is.function)))) warning('probably provided wrong type of function argument')
-    ls <- lapply(ls, function(el) if(is.nf(el)) 'function' else el)
+    ##if(any(unlist(lapply(ls, is.function)))) warning('probably provided wrong type of function argument')
+    ls <- lapply(ls, function(el) if(is.nf(el) || is.function(el)) 'function' else el)   ## functions -> 'function'
     ls2 <- list()
+    defaultOptions <- getNimbleOption('MCMCcontrolDefaultList')
     for(i in seq_along(ls)) {
-        if(length(ls[[i]]) > 0) {
-            deparsedItem <- deparse(ls[[i]])
-            if(length(deparsedItem) > 1) deparsedItem <- paste0(deparsedItem, collapse='')
-            ls2[[i]] <- paste0(names(ls)[i], ': ', deparsedItem)
-        }
+        controlName <- names(ls)[i]
+        controlValue <- ls[[i]]
+        if(length(controlValue) == 0) next   ## remove length 0
+        if(controlName %in% names(defaultOptions))   ## skip default control values
+            if(identical(controlValue, defaultOptions[[controlName]])) next
+        deparsedItem <- deparse(controlValue)
+        if(length(deparsedItem) > 1) deparsedItem <- paste0(deparsedItem, collapse='')
+        ls2[[i]] <- paste0(controlName, ': ', deparsedItem)
     }
     ls2 <- ls2[unlist(lapply(ls2, function(i) !is.null(i)))]
     str <- paste0(ls2, collapse = ',  ')
+    ##if(length(ls2) == 1)
+    ##    str <- paste0(str, ', default')
     str <- gsub('\"', '', str)
     str <- gsub('c\\((.*?)\\)', '\\1', str)
     return(str)
