@@ -1,9 +1,14 @@
 nimKeyWords <- list(copy = 'nimCopy',
                     print = 'nimPrint',
+                    cat = 'nimCat',
                     step = 'nimStep',
                     equals = 'nimEquals',
                     dim = 'nimDim',
                     stop = 'nimStop',
+                    numeric = 'nimNumeric',
+                    integer = 'nimInteger',
+                    matrix = 'nimMatrix',
+                    array = 'nimArray',
                     round = 'nimRound')
 
 nfMethodRC <- 
@@ -21,7 +26,7 @@ nfMethodRC <-
                     initialize = function(method, name) {
                     	
                         if(!missing(name)) uniqueName <<- name ## only needed for a pure RC function. Not needed for a nimbleFunction method
-						neededRCfuns <<- list()	
+                        neededRCfuns <<- list()	
                         argInfo <<- formals(method)
                         code <<- nf_changeNimKeywords(body(method))  ## changes all nimble keywords, e.g. 'print' to 'nimPrint'; see 'nimKeyWords' list at bottom
                         if(code[[1]] != '{')  code <<- substitute({CODE}, list(CODE=code))
@@ -60,6 +65,8 @@ nfMethodRC <-
                             returnTypeDeclaration <- code[[returnLineNum]][[2]]
                             code[returnLineNum] <<- NULL
                         }
+                        ## a very patchy solution: switch nimInteger back to integer
+                        if(as.character(returnTypeDeclaration[[1]]) == 'nimInteger') returnTypeDeclaration[[1]] <- as.name('integer')
                         returnType <<- returnTypeDeclaration
                     },
                     generateFunctionObject = function(keep.nfMethodRC = FALSE) {
@@ -93,13 +100,15 @@ nf_changeNimKeywords <- function(code){
 nf_changeNimKeywordsOne <- function(code){
     if(length(code) == 1){
         if(as.character(code) %in% names(nimKeyWords) ) {
-            if(is.call(code)) 
+            if(is.call(code)) {
                 code[[1]] <- as.name( nimKeyWords[[as.character(code)]] )
-            else
-                code <- as.name( nimKeyWords[[as.character(code)]] )
+            } else {
+                if(!is.character(code))
+                    code <- as.name( nimKeyWords[[as.character(code)]] )
+            }
         }
     }
-    else if(length(code) > 1){	
+    else if(length(code) > 1){ 
         for(i in seq_along(code) ) {
             if(!is.null(code[[i]]) )
                 code[[i]] <- nf_changeNimKeywordsOne(code[[i]])
