@@ -25,7 +25,7 @@ distributionsInputList <- list(
     dbin    = list(BUGSdist = 'dbin(prob, size)',
                    Rdist    = 'dbinom(size, prob)',
                    discrete = TRUE,
-                   range = c(0, 1),
+                   range = c(0, Inf),
                    pqAvail = TRUE,
                    alias   = 'dbinom'),
     
@@ -47,13 +47,6 @@ distributionsInputList <- list(
                          range    = c(0, Inf),
                          discrete = TRUE),
 
-    dmulti  = list(BUGSdist = 'dmulti(prob, size)',
-                   Rdist    = 'dmulti(size, prob)',
-                   types    = c('value = double(1)', 'prob = double(1)'),
-                   range    = c(0, Inf),
-                   discrete = TRUE,
-                   alias    = 'dmultinom'),
-    
     dnegbin = list(BUGSdist = 'dnegbin(prob, size)',
                    Rdist    = 'dnbinom(size, prob)',
                    discrete = TRUE,
@@ -74,7 +67,7 @@ distributionsInputList <- list(
     
     dbeta   = list(BUGSdist = 'dbeta(shape1, shape2, mean, sd)',
                    Rdist    = 'dbeta(shape1 = mean^2*(1-mean)/sd^2-mean, shape2 = mean*(1-mean)^2/sd^2+mean-1)',
-                   altParams= c('mean = shape1/(shape1+shape2)', 'sd = sqrt(shape1*shape2/((shape1*shape2)^2*(shape1+shape2+1)))'),
+                   altParams= c('mean = shape1/(shape1+shape2)', 'sd = sqrt(shape1*shape2/((shape1 + shape2)^2*(shape1+shape2+1)))'),
                    range    = c(0, 1),
                    pqAvail  = TRUE),
     
@@ -152,15 +145,19 @@ distributionsInputList <- list(
                    Rdist    = c('dmnorm_chol(mean, cholesky = chol(prec), prec_param = 1)', 'dmnorm_chol(mean, cholesky = chol(cov), prec_param = 0)', 'dmnorm_chol(mean, cholesky, prec_param)'),
                    altParams= c('prec = calc_dmnormAltParams(cholesky, prec_param, 1)', 'cov = calc_dmnormAltParams(cholesky, prec_param, 0)'),
                    types    = c('value = double(1)', 'mean = double(1)', 'cholesky = double(2)', 'prec = double(2)', 'cov = double(2)')),
-    ## altParams above is inefficient computationally;  want back/forwardsolve to avoid inverse; and having crossProd would be more efficient too - CP
-    ## also at the moment, the cholesky arg being fed to calc_dmnormAltParams is chol(Q) rather than lifted_chol_foo; we should revisit this when we rework altParms
-    ## calculations involved with dmnorm:
-    ## parametrized using   Covariance (S)                 Precision (Q)
-    ## simulate:            L_S %*% b                      R_Q^-1 %*% b (backslv)
-    ## calculate:           L_S^-1 %*% y (fwdslv)          R_Q %*% y
-    ## conjugate update:    S^-1, or  L_S^-1 %*% I ???     Q
     
-    ## dmt     = list(BUGSdist = 'dmt(mu, T, k)'),   ## not sure the state of this?  -DT
+    dmulti  = list(BUGSdist = 'dmulti(prob, size)',
+                   Rdist    = 'dmulti(size, prob)',
+                   types    = c('value = double(1)', 'prob = double(1)'),
+                   range    = c(0, Inf),
+                   discrete = TRUE,
+                   alias    = 'dmultinom'),
+    
+    dmvt  = list(BUGSdist = 'dmvt(mu, prec, scale, cholesky, df, prec_param)',
+                   Rdist    = c('dmvt_chol(mu, cholesky = chol(prec), df = df, prec_param = 1)', 'dmvt_chol(mu, cholesky = chol(scale), df = df, prec_param = 0)', 'dmvt_chol(mu, cholesky, df = df, prec_param)'),
+                   ## I think it's okay to use the same altParams function for multivariate normal and t
+                   altParams= c('prec = calc_dmnormAltParams(cholesky, prec_param, 1)', 'scale = calc_dmnormAltParams(cholesky, prec_param, 0)'),
+                   types    = c('value = double(1)', 'mu = double(1)', 'cholesky = double(2)', 'df = double(0)', 'prec = double(2)', 'scale = double(2)')),
     
     dwish   = list(BUGSdist = 'dwish(R, df, S)',
                    Rdist    = c('dwish_chol(cholesky = chol(R), df, scale_param = 0)', 'dwish_chol(cholesky = chol(S), df, scale_param = 1)'),
