@@ -325,6 +325,9 @@ ndf_generateGetParamSwitchFunction <- function(typesListAll, paramIDs, type, nDi
     paramIDs <- as.integer(paramIDs)
     answerAssignmentExpressions <- lapply(typesListAll, function(x) substitute(PARAMANSWER_ <- ANSEXPR, list(ANSEXPR = x)))
     switchCode <- as.call(c(list(quote(nimSwitch), quote(PARAMID_), paramIDs), answerAssignmentExpressions))
+    # avoid arg name mismatch based on R partial arg name matching
+    names(answerAssignmentExpressions) <- NULL
+    names(switchCode)[2:3] <- c('paramID', 'IDoptions')
     if(nDim == 0) {
         answerInitCode <- quote(PARAMANSWER_ <- 0)  ## this avoids a Windows compiler warning about a possibly unassigned return variable
         ans <- try(eval(substitute(
